require 'erb'

module LaTeXo
  TRANS = Hash[*(%w(
	= $=$ | $|$ < $<$ > $>$ # \# $ \$ % \% & \& ~ \~{}
	_ \_ ^ \^{} { \{ } \} \\ $\backslash$
        ))]
  TRANS_M = Hash[*(%w(
	= $=$ | $|$ < $<$ > $>$ # \# # \# $ \$ % \% & \& * $*$
	~ {\tt\symbol{126}} _ \_ ^ {\tt\symbol{94}}
	{ \{ } \} [ $[ ] $] \\ $\backslash$ * \*
	))]

  TRANS_REG = Regexp.new("[#{Regexp.escape(TRANS.keys.join)}]")
  TRANS_M_REG = Regexp.new("[#{Regexp.escape(TRANS_M.keys.join)}]")

  def convstr(str)
    quote(str)
  end

  def quote(str, quotable=false)
    if quotable
      str.gsub(TRANS_REG) do |m|
	TRANS[m]
      end
    else
      str.gsub(TRANS_M_REG) do |m|
	TRANS_M[m]
      end
    end
  end

  module_function :convstr
  module_function :quote
end

class Plain
  def initialize(env)
    @env = env
  end

  def feed(line)
    case line
    when '*Abstract'
      Abstract.new(@env).feed(line)
    when /^\*/
      Section.new(@env).feed(line)
    when ">||"
      Code.new(@env).feed(line)
    when /^\-/
      Itemize.new(@env).feed(line)
    when /^\+/
      Enumerate.new(@env).feed(line)
    when /^\:/
      Description.new(@env).feed(line)
    when '<img src="http://www.druby.org/ijpp/d2block.jpg" />'
      @env.puts(FigureD2Block.new.to_s)
      self
    when '<img src="http://www2a.biglobe.ne.jp/~seki/ruby/d2front3.jpg" />'
      @env.puts(FigureD2Front.new.to_s)
      self
    else
      @env.puts(to_tex(line))
      self
    end
  end

  def to_tex(line)
    line.split(/(\(\()|(\)\))/).collect do |str|
      case str
      when '(('
        '\footnote{'
      when '))'
        '}'
      else
        LaTeXo.convstr(str)
      end
    end.join('')
  end
end

class Itemize
  def initialize(env)
    @env = env
    @first = true
  end

  def feed(line)
    case line
    when /^\-\s*(.*)/
      if @first
        @env.puts('\begin{itemize}')
        @first = false
      end
      @env.puts('\item ' + LaTeXo.convstr($1))
      self
    else
      @env.puts('\end{itemize}')
      Plain.new(@env)
    end
  end
end

class Enumerate
  def initialize(env)
    @env = env
    @first = true
  end

  def feed(line)
    case line
    when /^\+\s*(.*)/
      if @first
        @env.puts('\begin{enumerate}')
        @first = false
      end
      @env.puts('\item ' + LaTeXo.convstr($1))
      self
    else
      @env.puts('\end{enumerate}')
      Plain.new(@env)
    end
  end
end

class Description
  def initialize(env)
    @env = env
    @first = true
  end

  def feed(line)
    case line
    when /^\:(.*)\s+\:(.*)/
      if @first
        @env.puts('\begin{description}')
        @first = false
      end
      @env.puts("\\item[#{LaTeXo.convstr($1)}] #{LaTeXo.convstr($2)}")
      self
    else
      @env.puts('\end{description}')
      Plain.new(@env)
    end
  end
end

class Abstract
  def initialize(env)
    @env = env
  end

  def feed(line)
    case line
    when '*Abstract'
      @env.puts('\begin{abstract}')
      self
    when /^\*/
      @env.puts('\end{abstract}')
      @env.puts
      Section.new(@env).feed(line)
    else
      @env.puts(LaTeXo.convstr(line))
      self
    end
  end
end

class Section
  Command = %w(section subsection subsubsection)

  def initialize(env)
    @env = env
  end

  def feed(line)
    if /^(\*+)([1-9]\.)*\s*(.*)/ =~ line
      put_section($1.size, $3)
    else
      raise(line)
    end
    Plain.new(@env)
  end

  def put_section(depth, caption)
    @env.puts("\\#{Command[depth - 1]}{#{LaTeXo.convstr(caption)}}")
  end
end

class Code
  def initialize(env)
    @env = env
  end

  def feed(line)
    case line
    when ">||"
      @env.puts('\small{')
      @env.puts('\begin{verbatim}')
      self
    when "||<"
      @env.puts('\end{verbatim}')
      @env.puts('}')
      Plain.new(@env)
    else
      @env.puts(line)
      self
    end
  end
end

class HaTeX
  def initialize(output=$stdout)
    @status = Plain.new(self)
    @label = {}
    @out = output
  end
  attr_reader :label, :out

  def feed(line)
    @status = @status.feed(line)
  end

  def puts(line='')
    @out.puts(line)
  end
end

class FigureD2Block
  def to_s
    <<'EOS'
\begin{figure}
% Use the relevant command for your figure-insertion program
% to insert the figure file.
% For example, with the option graphics use
\resizebox{0.60\textwidth}{!}{%
  \includegraphics{d2block.eps}
}
% If not, use
%\vspace{5cm}       % Give the correct figure height in cm
\caption{dRuby application software structure}
\label{fig:d2block}       % Give a unique label
\end{figure}
EOS
  end
end

class FigureD2Front
  def to_s
    <<'EOS'
\begin{figure}
% Use the relevant command for your figure-insertion program
% to insert the figure file.
% For example, with the option graphics use
\resizebox{0.90\textwidth}{!}{%
  \includegraphics{d2front3.eps}
}
% If not, use
%\vspace{5cm}       % Give the correct figure height in cm
\caption{front object and typical system}
\label{fig:d2front3}       % Give a unique label
\end{figure}
EOS
  end
end

class HaTeXIJPP
  def initialize(fname = 'hatex.rtex')
    @erb = ERB.new(File.read(fname))
    @body = []
  end
  
  def convert
    hatex = HaTeX.new(self)
    while line = gets
      hatex.feed(line.chomp)
    end
    print @erb.result(binding)
  end

  def puts(line='')
    @body.push(line + "\n")
  end
end



##
while line = gets
  break if line.index("($Revision:") == 0
end

HaTeXIJPP.new.convert


